/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors: MIP Holdings (Pty) Ltd ("MIP")                       *
*               PSC                                                  *
*                                                                    *
*********************************************************************/

TRIGGER PROCEDURE FOR CREATE OF ryc_page_object .

/* Created automatically using ERwin ICF Trigger template db/af/erw/afercustrg.i
   Do not change manually. Customisations to triggers should be placed in separate
   include files pulled into the trigger. ICF auto generates write trigger custom
   include files. Delete or create customisation include files need to be created
   manually. Be sure to put the hook in ERwin directly so as not to have you changes
   overwritten.
   User defined Macro (UDP) Summary (case sensitive)
   ryc_page_object           Expands to full table name, e.g. gsm_user
   %TableFLA            Expands to table unique code, e.g. gsmus
   %TableObj            If blank or not defined expands to table_obj with no prefix (framework standard)
                        If defined, uses this field as the object field
                        If set to "none" then indicates table does not have an object field
   XYZ                  Do not define so we can compare against an empty string

   See docs for explanation of replication macros 
*/   

&SCOPED-DEFINE TRIGGER_TABLE ryc_page_object
&SCOPED-DEFINE TRIGGER_FLA rycpo
&SCOPED-DEFINE TRIGGER_OBJ page_object_obj


DEFINE BUFFER lb_table FOR ryc_page_object.      /* Used for recursive relationships */
DEFINE BUFFER lb1_table FOR ryc_page_object.     /* Used for lock upgrades */

DEFINE BUFFER o_ryc_page_object FOR ryc_page_object.

/* Standard top of CREATE trigger code */
{af/sup/aftrigtopc.i}

  



/* Generated by ICF ERwin Template */
/* ryc_smartobject contains ryc_page_object ON CHILD INSERT RESTRICT */
IF 
    ( ryc_page_object.container_smartobject_obj <> 0 ) THEN
  DO:
    IF NOT(CAN-FIND(FIRST ryc_smartobject WHERE
        ryc_page_object.container_smartobject_obj = ryc_smartobject.smartobject_obj)) THEN
        DO:
          /* Cannot create child because parent does not exist ! */
          ASSIGN lv-error = YES lv-errgrp = "AF ":U lv-errnum = 102 lv-include = "ryc_page_object|ryc_smartobject":U.
          RUN error-message (lv-errgrp, lv-errnum, lv-include).
        END.
    
    
  END.

/* Generated by ICF ERwin Template */
/* ryc_page contains ryc_page_object ON CHILD INSERT RESTRICT */
IF 
    ( ryc_page_object.page_obj <> 0 ) THEN
  DO:
    IF NOT(CAN-FIND(FIRST ryc_page WHERE
        ryc_page_object.page_obj = ryc_page.page_obj)) THEN
        DO:
          /* Cannot create child because parent does not exist ! */
          ASSIGN lv-error = YES lv-errgrp = "AF ":U lv-errnum = 102 lv-include = "ryc_page_object|ryc_page":U.
          RUN error-message (lv-errgrp, lv-errnum, lv-include).
        END.
    
    
  END.

/* Generated by ICF ERwin Template */
/* ryc_object_instance is on ryc_page_object ON CHILD INSERT RESTRICT */
IF 
    ( ryc_page_object.object_instance_obj <> 0 ) THEN
  DO:
    IF NOT(CAN-FIND(FIRST ryc_object_instance WHERE
        ryc_page_object.object_instance_obj = ryc_object_instance.object_instance_obj)) THEN
        DO:
          /* Cannot create child because parent does not exist ! */
          ASSIGN lv-error = YES lv-errgrp = "AF ":U lv-errnum = 102 lv-include = "ryc_page_object|ryc_object_instance":U.
          RUN error-message (lv-errgrp, lv-errnum, lv-include).
        END.
    
    
  END.






ASSIGN ryc_page_object.{&TRIGGER_OBJ} = getNextObj().







/* Update Audit Log */
IF CAN-FIND(FIRST gsc_entity_mnemonic
            WHERE gsc_entity_mnemonic.entity_mnemonic = 'rycpo':U
              AND gsc_entity_mnemonic.auditing_enabled = YES) THEN
  RUN af/app/afauditlgp.p (INPUT "CREATE":U, INPUT "rycpo":U, INPUT BUFFER ryc_page_object:HANDLE, INPUT BUFFER o_ryc_page_object:HANDLE).

/* Standard bottom of CREATE trigger code */
{af/sup/aftrigendc.i}


/* Place any specific CREATE trigger customisations here */
